@@ -4,9 +4,10 @@ import threading
 __author__ = 'leifj'
 
 from xmlsec.exceptions import XMLSigException
-from urlparse import urlparse
+from six.moves.urllib_parse import urlparse
 import os
 import logging
+from xmlsec.utils import b642pem
 
 _modules = {}
 
@@ -16,7 +17,7 @@ try:
 except ImportError:
     raise XMLSigException("pykcs11 is required for PKCS#11 keys - cf README.rst")
 
-all_attributes = PyKCS11.CKA.keys()
+all_attributes = list(PyKCS11.CKA.keys())
 
 # remove the CKR_ATTRIBUTE_SENSITIVE attributes since we can't get
 all_attributes.remove(PyKCS11.LowLevel.CKA_PRIVATE_EXPONENT)
@@ -35,7 +36,7 @@ def parse_uri(pk11_uri):
 
     logging.debug("parsed pkcs11 uri: %s" % repr(o))
 
-    slot = 0
+    slot = None
     library = None
     keyname = None
     query = {}
@@ -80,7 +81,7 @@ def parse_uri(pk11_uri):
 
 
 def _intarray2bytes(x):
-    return ''.join(chr(i) for i in x)
+    return bytearray(x)
 
 
 def _close_session(session):
@@ -116,19 +117,6 @@ def _get_object_attributes(session, o):
     return dict(zip(all_attributes, attributes))
 
 
-def _cert_der2pem(der):
-    x = base64.standard_b64encode(der)
-    r = "-----BEGIN CERTIFICATE-----\n"
-    while len(x) > 64:
-        r += x[0:64]
-        r += "\n"
-        x = x[64:]
-    r += x
-    r += "\n"
-    r += "-----END CERTIFICATE-----"
-    return r
-
-
 def _find_key(session, keyname):
     key = _find_object(session, [(CKA_LABEL, keyname), (CKA_CLASS, CKO_PRIVATE_KEY), (CKA_KEY_TYPE, CKK_RSA)])
     if key is None:
@@ -138,7 +126,7 @@ def _find_key(session, keyname):
     cert_pem = None
     if cert is not None:
         cert_a = _get_object_attributes(session, cert)
-        cert_pem = _cert_der2pem(_intarray2bytes(cert_a[CKA_VALUE]))
+        cert_pem = b642pem(base64.standard_b64encode(_intarray2bytes(cert_a[CKA_VALUE])))
         logging.debug(cert)
     return key, cert_pem
 
@@ -146,19 +134,38 @@ def _find_key(session, keyname):
 _session_lock = threading.RLock()
 
 
-def _session(library, slot, pin=None):
+def _session(library, slot=None, pin=None, pk11_uri=None):
     _session_lock.acquire()
+
+    # XXX: adhoc fix -- should test cases where slot, pin and pk11_uri
+    # contradict or both are 'None'
+    if slot is None and pk11_uri is not None:
+        library, slot, keyname, query = parse_uri(pk11_uri)
+        pin_spec = query.get('pin', "env:PYKCS11PIN")
+        if pin_spec.startswith("env:"):
+            pin = os.environ.get(pin_spec[4:], None)
+        else:
+            pin = pin_spec
+
     if not library in _modules:
         logging.debug("loading library %s" % library)
         lib = PyKCS11.PyKCS11Lib()
         assert type(library) == str  # lib.load does not like unicode
         lib.load(library)
+        # XXX should check result of C_Initialize()
         lib.lib.C_Initialize()
         _modules[library] = lib
     else:
         logging.debug("already loaded: %s: %s" % (library, _modules[library]))
 
     lib = _modules[library]
+
+    # XXX: adhoc fix: if no slot given, use the first in the list
+    # (not the one named '0')
+    # Should be replaced by some proper pkcs11 uri search
+    if slot is None:
+        slot = lib.getSlotList(tokenPresent=True)[0]
+
     session = lib.openSession(slot)
     if pin is not None:
         assert type(pin) == str  # session.login does not like unicode
