@@ -6,8 +6,9 @@ __author__ = 'leifj'
 
 import os
 import pkg_resources
-import lxml.etree as etree
-from StringIO import StringIO
+from defusedxml import lxml
+from lxml import etree
+from six.moves import StringIO
 import xmlsec
 
 
@@ -21,7 +22,15 @@ class XMLTestData():
         self.name = name
         self.data = {}
         for fn in pkg_resources.resource_listdir(self.base, self.name):
-            self.data[fn] = pkg_resources.resource_stream(self.base, "%s/%s" % (self.name, fn)).read()
+            if fn.endswith(".xml"):
+                with pkg_resources.resource_stream(self.base, "%s/%s" % (self.name, fn)) as data:
+                    self.data[fn] = data.read()
+
+    def has_data(self, n):
+        return n in self.data
+
+    def __str__(self):
+        return "Testcase {}/{}".format(self.base, self.name)
 
     def as_buf(self, n):
         assert n in self.data, XMLTestDataException("No data named %s in test case %s" % (n, self.name))
