@@ -1,14 +1,21 @@
+from defusedxml import lxml
 from lxml import etree
-
-__author__ = 'ft'
-
 import os
 import copy
 import unittest
 import xmlsec
 import pkg_resources
 from xmlsec.test.case import load_test_data
-from xmlsec import constants
+from xmlsec import constants, utils
+import six
+import base64
+from . import find_alts, run_cmd
+import tempfile
+
+__author__ = 'ft'
+
+XMLSEC1 = find_alts(['/usr/local/bin/xmlsec1', '/usr/bin/xmlsec1'])
+
 
 def root(t):
     if hasattr(t, 'getroot') and hasattr(t.getroot, '__call__'):
@@ -22,17 +29,87 @@ def _get_all_signatures(t):
     for sig in t.findall(".//{%s}Signature" % xmlsec.NS['ds']):
         sv = sig.findtext(".//{%s}SignatureValue" % xmlsec.NS['ds'])
         assert sv is not None
-        # base64-dance to normalize newlines
-        res.append(sv.decode('base64').encode('base64'))
+        # normalize newlines
+        sv = ''.join(sv.split('\n'))
+        if not isinstance(sv, six.binary_type):
+            sv = bytes(sv, encoding='utf-8')
+        res.append(sv)
     return res
 
 
-class TestTransforms(unittest.TestCase):
+class TestSignVerifyXmlSec1(unittest.TestCase):
     def setUp(self):
         self.datadir = pkg_resources.resource_filename(__name__, 'data')
         self.private_keyspec = os.path.join(self.datadir, 'test.key')
         self.public_keyspec = os.path.join(self.datadir, 'test.pem')
+        self.cases = load_test_data('data/verifyxmlsec1')
+        self.tmpf = tempfile.NamedTemporaryFile(delete=False)
 
+    @unittest.skipIf(XMLSEC1 is None, "xmlsec1 binary not installed")
+    def test_sign_verify_all(self):
+        """
+        Run through all testcases, sign and verify using xmlsec1
+        """
+        for case in self.cases.values():
+            if case.has_data('in.xml'):
+                signed = xmlsec.sign(case.as_etree('in.xml'),
+                                     key_spec=self.private_keyspec,
+                                     cert_spec=self.public_keyspec)
+                res = xmlsec.verify(signed, self.public_keyspec)
+                self.assertTrue(res)
+                with open(self.tmpf.name, "w") as fd:
+                    xml_str = utils.etree_to_string(signed)
+                    fd.write(xml_str)
+
+                run_cmd([XMLSEC1,
+                         '--verify',
+                         '--store-references',
+                         '--id-attr:ID', 'urn:oasis:names:tc:SAML:2.0:metadata:EntityDescriptor',
+                         '--id-attr:ID', 'urn:oasis:names:tc:SAML:2.0:metadata:EntitiesDescriptor',
+                         '--id-attr:ID', 'urn:oasis:names:tc:SAML:2.0:assertion:Assertion',
+                         '--verification-time', '2009-11-01 12:00:00',
+                         '--trusted-pem', self.public_keyspec,
+                         self.tmpf.name])
+
+    def tearDown(self):
+        if os.path.exists(self.tmpf.name):
+            pass
+            # os.unlink(self.tmpf.name)
+
+
+class TestVerify(unittest.TestCase):
+
+    def setUp(self):
+        self.datadir = pkg_resources.resource_filename(__name__, 'data')
+        self.resource_dir = pkg_resources.resource_filename(__name__, '')
+        self.cases = load_test_data('data/verify')
+
+    def test_verify_all(self):
+        for case in self.cases.values():
+            print(str(case))
+            public_keyspec = os.path.join(self.resource_dir, case.name, "signer.crt")
+            res = xmlsec.verify(case.as_etree("in.xml"), public_keyspec)
+            self.assertTrue(res)
+
+    def test_verify_one(self):
+        case = self.cases['it-connector']
+        public_keyspec = os.path.join(self.resource_dir, case.name, "signer.crt")
+        res = xmlsec.verify(case.as_etree("in.xml"), public_keyspec)
+        self.assertTrue(res)
+
+    def test_verify_pss_mgf1(self):
+        case = self.cases['eesti']
+        public_keyspec = os.path.join(self.resource_dir, case.name, "signer.crt")
+        res = xmlsec.verify(case.as_etree("in.xml"), public_keyspec)
+        self.assertTrue(res)
+
+
+class TestSignVerify(unittest.TestCase):
+    
+    def setUp(self):
+        self.datadir = pkg_resources.resource_filename(__name__, 'data')
+        self.private_keyspec = os.path.join(self.datadir, 'test.key')
+        self.public_keyspec = os.path.join(self.datadir, 'test.pem')
         self.cases = load_test_data('data/signverify')
 
     def test_sign_verify_SAML_assertion1(self):
@@ -123,8 +200,8 @@ class TestTransforms(unittest.TestCase):
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
@@ -144,11 +221,26 @@ class TestTransforms(unittest.TestCase):
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
+    def test_duo_vuln_attack(self):
+        """
+        Test https://duo.com/blog/duo-finds-saml-vulnerabilities-affecting-multiple-implementations
+        """
+        case = self.cases['SAML_assertion_sha256']
+        print("XML input :\n{}\n\n".format(case.as_buf('in.xml')))
+
+        signed = xmlsec.sign(case.as_etree('in.xml'),
+                             key_spec=self.private_keyspec,
+                             cert_spec=self.public_keyspec)
+        refs = xmlsec.verified(signed, self.public_keyspec)
+        self.assertTrue(len(refs) == 1)
+        print("verified XML: %s" % etree.tostring(refs[0]))
+        assert('evil' not in [x.text for x in refs[0].findall(".//{%s}AttributeValue" % 'urn:oasis:names:tc:SAML:2.0:assertion')])
+
     def test_verify_SAML_assertion1(self):
         """
         Test that we can verify signatures created by another implementation (xmlsec1).
@@ -168,18 +260,34 @@ class TestTransforms(unittest.TestCase):
 
         # modify the givenName in the XML and make sure the signature
         # does NOT validate anymore
-        case.data['out.xml'] = case.data['out.xml'].replace('>Bar<', '>Malory<')
+        case.data['out.xml'] = case.data['out.xml'].replace(b'>Bar<', b'>Malory<')
 
         print("XML input :\n{}\n\n".format(case.as_buf('out.xml')))
         with self.assertRaises(xmlsec.XMLSigException):
-            xmlsec.verify(case.as_etree('out.xml'), self.public_keyspec)
+            res = xmlsec.verify(case.as_etree('out.xml'), self.public_keyspec)
+            print(res)
+
+    def test_sign_xades(self):
+        """
+        Test that we can sign an already signed document without breaking the first signature
+        """
+
+        case = self.cases['dont_break_xades']
+        t = case.as_etree('in.xml')
+
+        signed = xmlsec.sign(t, self.private_keyspec)
+        self.assertIsNotNone(signed)
+        digests = [dv.text for dv in signed.findall('.//{%s}DigestValue' % xmlsec.NS['ds'])]
+        assert 'JvmW5vKjaTEVHzOdiC/H3HSGNocGamY9sDeU86ld6TA=' in digests
+        res = xmlsec.verify(signed, self.public_keyspec)
+        self.assertTrue(res)
 
     def test_mm1(self):
         case = self.cases['mm1']
         signed = xmlsec.sign(case.as_etree('in.xml'),
                              key_spec=self.private_keyspec,
                              cert_spec=self.public_keyspec)
-        print etree.tostring(signed)
+        print(etree.tostring(signed))
 
     def test_mm2(self):
         case = self.cases['mm2']
@@ -187,6 +295,8 @@ class TestTransforms(unittest.TestCase):
         xmlsec.add_enveloped_signature(t,
                                        pos=-1,
                                        c14n_method=constants.TRANSFORM_C14N_EXCLUSIVE,
+                                       digest_alg=constants.ALGORITHM_DIGEST_SHA1,
+                                       signature_alg=constants.ALGORITHM_SIGNATURE_RSA_SHA1,
                                        transforms=[constants.TRANSFORM_ENVELOPED_SIGNATURE])
         signed = xmlsec.sign(t,
                              key_spec=self.private_keyspec,
@@ -194,17 +304,17 @@ class TestTransforms(unittest.TestCase):
 
         expected = case.as_etree('out.xml')
 
-        print " --- Expected"
-        print etree.tostring(expected)
-        print " --- Actual"
-        print etree.tostring(signed)
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
 
         # extract 'SignatureValue's
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
@@ -217,17 +327,17 @@ class TestTransforms(unittest.TestCase):
 
         expected = case.as_etree('out.xml')
 
-        print " --- Expected"
-        print etree.tostring(expected)
-        print " --- Actual"
-        print etree.tostring(signed)
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
 
         # extract 'SignatureValue's
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
@@ -240,17 +350,17 @@ class TestTransforms(unittest.TestCase):
 
         expected = case.as_etree('out.xml')
 
-        print " --- Expected"
-        print etree.tostring(expected)
-        print " --- Actual"
-        print etree.tostring(signed)
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
 
         # extract 'SignatureValue's
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
@@ -260,6 +370,8 @@ class TestTransforms(unittest.TestCase):
         xmlsec.add_enveloped_signature(t,
                                        pos=-1,
                                        c14n_method=constants.TRANSFORM_C14N_EXCLUSIVE,
+                                       digest_alg=constants.ALGORITHM_DIGEST_SHA1,
+                                       signature_alg=constants.ALGORITHM_SIGNATURE_RSA_SHA1,
                                        transforms=[constants.TRANSFORM_ENVELOPED_SIGNATURE])
         signed = xmlsec.sign(t,
                              key_spec=self.private_keyspec,
@@ -267,17 +379,17 @@ class TestTransforms(unittest.TestCase):
 
         expected = case.as_etree('out.xml')
 
-        print " --- Expected"
-        print etree.tostring(expected)
-        print " --- Actual"
-        print etree.tostring(signed)
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
 
         # extract 'SignatureValue's
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEqual(signed_sv, expected_sv)
 
@@ -289,6 +401,8 @@ class TestTransforms(unittest.TestCase):
         xmlsec.add_enveloped_signature(t,
                                        pos=-1,
                                        c14n_method=constants.TRANSFORM_C14N_EXCLUSIVE,
+                                       digest_alg=constants.ALGORITHM_DIGEST_SHA1,
+                                       signature_alg=constants.ALGORITHM_SIGNATURE_RSA_SHA1,
                                        transforms=[constants.TRANSFORM_ENVELOPED_SIGNATURE])
         signed = xmlsec.sign(t,
                              key_spec=self.private_keyspec,
@@ -300,33 +414,54 @@ class TestTransforms(unittest.TestCase):
         sig = t.find("./{%s}Signature" % xmlsec.NS['ds'])
         digest = sig.findtext('.//{%s}DigestValue' % xmlsec.NS['ds'])
 
-        print " --- Expected digest value"
-        print expected_digest
-        print " --- Actual digest value"
-        print digest
+        print(" --- Expected digest value")
+        print(expected_digest)
+        print(" --- Actual digest value")
+        print(digest)
 
-        print " --- Expected"
-        print etree.tostring(expected)
-        print " --- Actual"
-        print etree.tostring(signed)
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
 
         # extract 'SignatureValue's
         expected_sv = _get_all_signatures(expected)
         signed_sv = _get_all_signatures(signed)
 
-        print "Signed   SignatureValue: %s" % (repr(signed_sv))
-        print "Expected SignatureValue: %s" % (repr(expected_sv))
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
 
         self.assertEquals(digest, expected_digest)
         self.assertEqual(signed_sv, expected_sv)
 
     def test_verify_href(self):
         case = self.cases['href']
-        t = case.as_etree('href.xml',remove_comments=False,remove_whitespace=False)
+        t = case.as_etree('href.xml', remove_comments=False, remove_whitespace=False)
         href_signer = os.path.join(self.datadir, "signverify/href/href-metadata-signer-2011.crt")
         res = xmlsec.verify(t, href_signer)
         self.assertTrue(res)
 
+    def test_edugain_with_xmlsec1(self):
+        case = self.cases['edugain']
+        t = case.as_etree('xmlsec1_in.xml')
+        signed = xmlsec.sign(t,
+                             key_spec=self.private_keyspec,
+                             cert_spec=self.public_keyspec)
+
+        expected = case.as_etree('xmlsec1_out.xml')
+
+        print(" --- Expected")
+        print(etree.tostring(expected))
+        print(" --- Actual")
+        print(etree.tostring(signed))
+
+        # extract 'SignatureValue's
+        expected_sv = _get_all_signatures(expected)
+        signed_sv = _get_all_signatures(signed)
+
+        print("Signed   SignatureValue: %s" % (repr(signed_sv)))
+        print("Expected SignatureValue: %s" % (repr(expected_sv)))
+
 
 def main():
     unittest.main()
